'''
Created on Apr 6, 2017

@author: breno
'''

import os
import logging
import sys
import pprint

from coverage_info import get_coverage_info

sys.path.insert(0, '/home/breno/workspace/ImportanceSampling')
from select_svn import get_entities_imp


def get_similarity_between_global_and_op(version_dir, target_iter, entity, hml_proportion):
    target_iter_dir = os.path.join(version_dir, 'covrel_generation', 'iter{}'.format(target_iter))
                                   
    def _get_covered_entities_from_spectrum_file(info_file):
        spectrum_file = os.path.join(target_iter_dir, info_file)
        covered_aux = get_coverage_info(spectrum_file, format_=['covered'])
        covered = covered_aux['{}_{}'.format('covered', entity)]
        return covered
    
    def _get_coveredcount_entities_from_spectrum_file(info_file):
        spectrum_file = os.path.join(target_iter_dir, info_file)
        covered_aux = get_coverage_info(spectrum_file, format_=['coveredcount'])
        covered = covered_aux['{}_{}'.format('coveredcount', entity)]
        return covered
    
    sim_dict = {}
    
    # Global
    global_spectrum = "ALL_iter{}.info".format(target_iter)
    covered_global = _get_coveredcount_entities_from_spectrum_file(global_spectrum)
    importance_groups = get_entities_imp(covered_global, hml_proportion)
    logging.debug("importance_groups: {}".format(importance_groups))
    logging.debug("importance_groups: H={}, M={}, L={}".format(
                    len(importance_groups['high']), len(importance_groups['medium']), len(importance_groups['low'])))
    covered_global_low = importance_groups['low']
    logging.debug("Total # entities covered in the global spectrum: {}".format(len(covered_global)))
    logging.debug("Total # 'low' entities in the global spectrum: {}".format(len(covered_global_low)))
    #pprint.pprint(covered_global_low)
    #logging.debug("BLA: {}".format(pprint.pprint(covered_global_low)))
    
    # OP
    op_spectra = [f for f in os.listdir(target_iter_dir) 
                  if f.startswith("OP") and f.endswith(".info") and 'iter{}'.format(target_iter) in f
                  and 'aux' not in f]
    for op_spectrum in op_spectra:
        #op = int(op_spectrum.replace('OP_','').replace('.info',''))
        op = int(op_spectrum.replace('OP','').replace('.info','').replace('_aux','').replace('_iter{}'.format(target_iter),''))
        covered_op = _get_covered_entities_from_spectrum_file(op_spectrum)
        intersection = set(covered_global_low).intersection(set(covered_op)) 
        #union = set(covered_global_low).union(set(covered_op))
        union = covered_global_low
        logging.debug("intersection ({}): {}".format(len(intersection), intersection))
        logging.debug("union ({}): {}".format(len(union), union))
        jaccard = len(intersection)/float(len(union))
        sim_dict[op] = jaccard
        logging.debug("jaccard: {}".format(jaccard))
        
    assert len(op_spectra) == len(sim_dict), "Verify op_spectra list! len(op_spectra) = {}, len(sim_dict) = {}".format(
                                len(op_spectra), len(sim_dict))
    
    return sim_dict
                
def get_tc_distribution_for_next_iter(budget, sim_dict):
    tc_for_next_iter = {}
    smaller, smaller_op = budget, None
    bigger, bigger_op = 0, None
    for op in sim_dict:
        proportion = sim_dict[op]/sum(sim_dict.values())
        num_tcs = int(round((budget * proportion), 0))
        tc_for_next_iter[op] = num_tcs
        #
        if num_tcs < smaller:
            smaller = num_tcs
            smaller_op = op
        if num_tcs >= bigger:
            bigger = num_tcs
            bigger_op = op
    
    if sum(tc_for_next_iter.values()) < budget:
        logging.debug("TC distribution does not match budget (due to rounding).\nBudget: {} | Dist: {}".format(budget, tc_for_next_iter))
        #print "smaller: ", smaller_op
        #print sum(tc_for_next_iter.values()), tc_for_next_iter
        diff = budget - sum(tc_for_next_iter.values())
        tc_for_next_iter[smaller_op] += diff
        
    elif sum(tc_for_next_iter.values()) > budget:
        logging.debug("TC distribution does not match budget (due to rounding).\nBudget: {} | Dist: {}".format(budget, tc_for_next_iter))
        #print "bigger"
        #print sum(tc_for_next_iter.values()), tc_for_next_iter
        diff = sum(tc_for_next_iter.values()) - budget
        tc_for_next_iter[bigger_op] -= diff
    
    logging.debug("smaller_op ({}): OP{} | bigger_op ({}): OP{}".format(smaller, smaller_op, bigger, bigger_op))
    return tc_for_next_iter
        
    


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    subject = 'gzip'
    version = 'v1'
    subject_dir = os.path.join('..', 'subjects', subject)
    version_dir = os.path.join(subject_dir, version)
    
    hml_proportion = (1/3.0, 1/3.0, 1/3.0)
    entity = 'function'
    budget = 100
    
    sim_dict = get_similarity_between_global_and_op()
    print "sim_dict: ", sim_dict
    
    tc_for_next_iter = get_tc_distribution_for_next_iter(budget, sim_dict)
    print "tc_for_next_iter: ", tc_for_next_iter
    

    
